/// Copyright (c) 2009 Ruben Penalva. All rights reserved.
/// Redistribution and use in source and binary forms, with or without modification, 
/// are permitted provided that the following conditions are met:
///    1. Redistributions of source code must retain the above copyright notice, this 
///    list of conditions and the following disclaimer.
///    2. Redistributions in binary form must reproduce the above copyright notice,
///    this list of conditions and the following disclaimer in the documentation and/or
///    other materials provided with the distribution.
///    3. The name of the author may not be used to endorse or promote products derived
///    from this software without specific prior written permission.
///
/// THIS SOFTWARE IS PROVIDED BY THE AUTHOR ``AS IS'' AND ANY EXPRESS OR IMPLIED WARRANTIES,
/// INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR 
/// A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE AUTHOR BE LIABLE FOR ANY DIRECT, 
/// INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED 
/// TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS 
/// INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT 
/// LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF 
/// THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.

/// @file Circle.h
/// This source file is part of 2dRender Delaunay Test.
/// The file holds the Circle class interface.
/// @author Ruben Penalva (ruben.penalva@rpenalva.com)
#ifndef _CIRCLE_H_
#define _CIRCLE_H_

#include "Point.h"

namespace DelaunayTest
{

/// @brief Represents a circle with radius and a center
class Circle
{
public:
	/// Center
	Point m_center;

	/// Radius
	float m_radius;

	/// Constructor
	/// Constructs a circle with center center and 
	/// radius radius
	/// @param center center of the circle
	/// @param radius radius of the circle
	Circle(const Point& center, float radius);

	/// Constructor
	/// Constructs a circle from three points
	/// @param p1 first circle defining point
	/// @param p2 second circle defining point
	/// @param p3 third circle defining point
	Circle(const Point& p1, const Point& p2, const Point& p3);

	/// Destructor
	~Circle(void);

private:
	/// @brief Computes radius and center of a circle 
	/// from three points
	/// @param p1 first circle defining point
	/// @param p2 second circle defining point
	/// @param p3 third circle defining point
	/// @param center output parameter being circle center
	/// @return radius of the circle
	float CircleFrom3Points(const Point& p1, const Point& p2, const Point& p3, Point& center);

	/// @brief Computes the 3x3 determinant
	/// @param m00 i0j0 matrix component 
	/// @param m01 i0j1 matrix component 
	/// @param m02 i0j2 matrix component 
	/// @param m10 i1j0 matrix component 
	/// @param m11 i1j1 matrix component 
	/// @param m12 i1j2 matrix component 
	/// @param m20 i2j0 matrix component 
	/// @param m21 i2j1 matrix component 
	/// @param m22 i2j2 matrix component 
	/// @return determinant
	float Determinant3x3(float m00, float m01, float m02, 
						 float m10, float m11, float m12, 
						 float m20, float m21, float m22);
};

}

#endif