/// Copyright (c) 2009 Ruben Penalva. All rights reserved.
/// Redistribution and use in source and binary forms, with or without modification, 
/// are permitted provided that the following conditions are met:
///    1. Redistributions of source code must retain the above copyright notice, this 
///    list of conditions and the following disclaimer.
///    2. Redistributions in binary form must reproduce the above copyright notice,
///    this list of conditions and the following disclaimer in the documentation and/or
///    other materials provided with the distribution.
///    3. The name of the author may not be used to endorse or promote products derived
///    from this software without specific prior written permission.
///
/// THIS SOFTWARE IS PROVIDED BY THE AUTHOR ``AS IS'' AND ANY EXPRESS OR IMPLIED WARRANTIES,
/// INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR 
/// A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE AUTHOR BE LIABLE FOR ANY DIRECT, 
/// INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED 
/// TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS 
/// INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT 
/// LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF 
/// THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.

/// @file DelaunayTest2dRender.cpp
/// This source file is part of 2dRender Delaunay Test.
/// The file holds the DelaunayTest2dRender class implementation.
/// @author Ruben Penalva (ruben.penalva@rpenalva.com)
#include "DelaunayTest2dRender.h"

#include <GL/glut.h>
#include <GL/gl.h>
#include <GL/glu.h>
#include <math.h>


using namespace DelaunayTest;

DelaunayTest2dRender::DelaunayTest2dRender(void)	:	m_depth(-10.0f), m_pointSize(5.0f), m_circleSegmentsCount(50)
{
}

DelaunayTest2dRender::~DelaunayTest2dRender(void)
{
}

void DelaunayTest2dRender::LoadPresetConfiguration(int width, int height)
{
	glClear(GL_COLOR_BUFFER_BIT | GL_DEPTH_BUFFER_BIT);
    glClearColor(0.5, 0.5, 0.7, 0.0);
    glColor3f(1.0, 1.0, 1.0);

	glEnable(GL_LINE_SMOOTH);
    glEnable(GL_CULL_FACE);
    glFrontFace(GL_CCW);

    glDisable(GL_DEPTH_TEST);
    glShadeModel(GL_SMOOTH);
	
	glViewport(0, 0, width, height);

	glMatrixMode(GL_PROJECTION);
	glLoadIdentity();

	// Calculate The Aspect Ratio Of The Window
	gluPerspective(45.0f,(GLfloat)width/(GLfloat)height, 0.1f, 100.0f);

	glMatrixMode(GL_MODELVIEW);	
	gluLookAt (0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, -1.0);  
	glLoadIdentity();

	glPointSize(m_pointSize);
}

void DelaunayTest2dRender::BeginFrame(void)
{
	glClear(GL_COLOR_BUFFER_BIT | GL_DEPTH_BUFFER_BIT);
}

void DelaunayTest2dRender::SetMaterial(float red, float green, float blue)
{
	glColor3f(red, green, blue);
}

void DelaunayTest2dRender::DrawPoints(const std::vector<Point>& points)
{
	int pointsCount = points.size();

    glBegin(GL_POINTS);
		for (int i = 0; i < pointsCount; ++i)
		{
			const Point& point = points[i];
			glVertex3f(point.m_x, point.m_y, m_depth);
		}
    glEnd();
}

void DelaunayTest2dRender::DrawLines(const std::vector<Line>& lines)
{
	int linesCount = lines.size();
	for (int i = 0; i < linesCount; ++i)
	{
		const Line& line = lines[i];

		glBegin(GL_LINES);
			glVertex3f(line.m_start.m_x, line.m_start.m_y, m_depth);
			glVertex3f(line.m_end.m_x, line.m_end.m_y, m_depth);	
		glEnd();
	}
}

void DelaunayTest2dRender::DrawCircles(const std::vector<Circle>& circles)
{
	int circlesCount = circles.size();

	for (int i = 0 ; i < circlesCount; ++i)
	{
		const Circle& circle = circles[i];
		float r = circle.m_radius;
		float cx = circle.m_center.m_x;
		float cy = circle.m_center.m_y;

		/// Draw a circle
		glBegin(GL_LINE_LOOP); 
			for (int j = 0; j < m_circleSegmentsCount; ++j) 
			{ 
				///get the current angle 
				float theta = 2.0f * 3.1415926f * j / float(m_circleSegmentsCount);

				//calculate the x component 
				float x = r * cosf(theta);

				//calculate the y component 
				float y = r * sinf(theta);

				//output vertex 
				glVertex3f(x + cx, y + cy, m_depth);
			} 
		glEnd(); 
	}
}

void DelaunayTest2dRender::EndFrame(void)
{
	glutSwapBuffers();
}