/// Copyright (c) 2009 Ruben Penalva. All rights reserved.
/// Redistribution and use in source and binary forms, with or without modification, 
/// are permitted provided that the following conditions are met:
///    1. Redistributions of source code must retain the above copyright notice, this 
///    list of conditions and the following disclaimer.
///    2. Redistributions in binary form must reproduce the above copyright notice,
///    this list of conditions and the following disclaimer in the documentation and/or
///    other materials provided with the distribution.
///    3. The name of the author may not be used to endorse or promote products derived
///    from this software without specific prior written permission.
///
/// THIS SOFTWARE IS PROVIDED BY THE AUTHOR ``AS IS'' AND ANY EXPRESS OR IMPLIED WARRANTIES,
/// INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR 
/// A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE AUTHOR BE LIABLE FOR ANY DIRECT, 
/// INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED 
/// TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS 
/// INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT 
/// LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF 
/// THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.

/// @file DelaunayTest2dRender.h
/// This source file is part of 2dRender Delaunay Test.
/// The file holds the DelaunayTest2dRender class interface.
/// @author Ruben Penalva (ruben.penalva@rpenalva.com)
#ifndef _DELAUNAY_2DRENDER_H_
#define _DELAUNAY_2DRENDER_H_

#include <vector>

#include "Point.h"
#include "Line.h"
#include "Circle.h"

namespace DelaunayTest
{

/// @brief  Opengl Render tailored specifically for 
///	2dRender test.
/// It can render these 2d geometry:
///		- Points
///		- Circles
///		- Lines
/// It only supports a material being a configurable 
///	emissive color.
class DelaunayTest2dRender
{
public:
	/// Constructor
	DelaunayTest2dRender(void);

	/// Destructor
	~DelaunayTest2dRender(void);

	/// @brief Loads a default opengl configuration
	/// @param width render window width
	/// @param height render window height
	void LoadPresetConfiguration(int width, int height);

	/// @brief Begins render frame
	void BeginFrame(void);

	/// @brief Sets the only material: emissive color
	/// @param red red channel
	/// @param green green channel
	/// @param blue blue channel
	void SetMaterial(float red, float green, float blue);

	/// @brief Draws a vector of points in 2d with a fixed
	/// point size
	/// @param points the vector of points
	void DrawPoints(const std::vector<Point>& points);

	/// @brief Draws a vector of lines in 2d
	/// @param lines the vector of lines
	void DrawLines(const std::vector<Line>& lines);

	/// @brief Draws a vector of circles in 2d
	/// @param circles the vector of circles
	void DrawCircles(const std::vector<Circle>& circles);

	/// @brief Ends render frame
	void EndFrame(void);

private:
	/// Depth at wich objects are
	const float m_depth;

	/// Fixed point size
	const float m_pointSize;

	/// Number of circle segments
	const int m_circleSegmentsCount;

	/// @note hide copy constructor from interface user
	DelaunayTest2dRender(const DelaunayTest2dRender&);

	/// @note hide assignment operator from interface user
	DelaunayTest2dRender& operator=(const DelaunayTest2dRender&);
};

}

#endif
