/// Copyright (c) 2009 Ruben Penalva. All rights reserved.
/// Redistribution and use in source and binary forms, with or without modification, 
/// are permitted provided that the following conditions are met:
///    1. Redistributions of source code must retain the above copyright notice, this 
///    list of conditions and the following disclaimer.
///    2. Redistributions in binary form must reproduce the above copyright notice,
///    this list of conditions and the following disclaimer in the documentation and/or
///    other materials provided with the distribution.
///    3. The name of the author may not be used to endorse or promote products derived
///    from this software without specific prior written permission.
///
/// THIS SOFTWARE IS PROVIDED BY THE AUTHOR ``AS IS'' AND ANY EXPRESS OR IMPLIED WARRANTIES,
/// INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR 
/// A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE AUTHOR BE LIABLE FOR ANY DIRECT, 
/// INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED 
/// TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS 
/// INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT 
/// LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF 
/// THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.

/// @file DelaunayTestLogic.h
/// This source file is part of 2dRender Delaunay Test.
/// The file holds the DelaunayTest2dRender class implementation.
/// @author Ruben Penalva (ruben.penalva@rpenalva.com)
#include "DelaunayTestLogic.h"

#include <time.h>
#include <algorithm>

#include "DelaunayTriangulation.h"
#include "QuadEdgeWalker.h"
#include "QuadEdge.h"
#include "Site.h"

using namespace DelaunayTest;

DelaunayTestLogic::DelaunayTestLogic(void)	:	m_pointsCount(5), m_minRange(-4.0f), 
												m_maxRange(4.0f), m_points2AddCount(5), 
												m_maxPoints2Add(30), m_isTriangulated(false),
												m_circlesEnabled(false)
{	
	srand(static_cast<unsigned int>(time(0)));

	Reset();
}

DelaunayTestLogic::~DelaunayTestLogic(void)
{
}
	
void DelaunayTestLogic::Triangulate(void)
{	
	if (m_isTriangulated)
	{
		return;
	}

	TriangulatePoints();

	m_isTriangulated = true;
}

void DelaunayTestLogic::Reset(void)
{
	/// Clear all the geometry
	m_points.clear();
	m_lines.clear();
	m_circles.clear();

	/// Randomize the points
	AddRandomPoints(m_pointsCount);

	m_isTriangulated = false;
	m_circlesEnabled = false;
}

void DelaunayTestLogic::AddSites(void)
{
	if (m_isTriangulated || (static_cast<int>(m_points.size()) + m_points2AddCount > m_maxPoints2Add))
	{
		return;
	}

	AddRandomPoints(m_points2AddCount);
}

void DelaunayTestLogic::RemoveSites(void)
{		
	int erasingIndex = m_points.size() - m_points2AddCount;
	if (m_isTriangulated || erasingIndex <= 0)
	{
		return;
	}

	m_points.erase(m_points.begin() + erasingIndex, m_points.end());
}

void DelaunayTestLogic::EnableCircles(void)
{
	m_circlesEnabled = !m_circlesEnabled;
}

const std::vector<Point>& DelaunayTestLogic::GetPoints(void) const
{
	return m_points;
}

const std::vector<Line>& DelaunayTestLogic::GetLines(void) const
{
	return m_lines;
}

const std::vector<Circle>& DelaunayTestLogic::GetCircles(void) const
{
	return m_circles;
}

bool DelaunayTestLogic::IsCirclesEnabled(void)
{
	return m_circlesEnabled;
}

bool DelaunayTestLogic::IsTriangulated(void)
{
	return m_isTriangulated;
}

void DelaunayTestLogic::AddRandomPoints(int pointsCount)
{
	for (int i = 0; i < pointsCount; ++i)
	{
		float x = (rand() / (static_cast<float>(RAND_MAX) + 1.0f)) * (m_maxRange - m_minRange) + m_minRange;
		float y = (rand() / (static_cast<float>(RAND_MAX) + 1.0f)) * (m_maxRange - m_minRange) + m_minRange;

		m_points.push_back(Point(x, y));
	}	
}

void DelaunayTestLogic::TriangulatePoints(void)
{
	/// Translate points to sites
	int pointsCount = m_points.size();
	std::vector<EdgeAlgebra::Site*> sites;
	for (int i = 0; i < pointsCount; ++i)
	{
		const Point& point = m_points[i];
		sites.push_back(new EdgeAlgebra::Site(point.m_x, point.m_y));
	}

	/// Sort the sites in order to run delaunay algorithm
	std::sort(sites.begin(), sites.end(), EdgeAlgebra::SiteLessThan);

	/// Make a delaunay triangulation
	EdgeAlgebra::DelaunayTriangulation delaunayTriangulation(sites);

	/// Get edges from delaunay triangulation
	EdgeAlgebra::QuadEdgeWalker walker(delaunayTriangulation.GetLeftHalve());
	std::map<EdgeAlgebra::QuadEdge*, char> triangulationEdges = walker.GetWalkedEdges();

	CreateLines(triangulationEdges);
	CreateCircles(triangulationEdges);

	/// Delete sites
	int sitesCount = sites.size();
	for (int i = 0; i < sitesCount; ++i)
	{
		delete sites[i];
	}
}

void DelaunayTestLogic::CreateLines(const std::map<EdgeAlgebra::QuadEdge*, char>& edges)
{
	std::map<EdgeAlgebra::QuadEdge*, char>::const_iterator it = edges.begin();
	std::map<EdgeAlgebra::QuadEdge*, char>::const_iterator itEnd = edges.end();
	while (it != itEnd)
	{			
		/// A line is composed of the edge origin and its destination
		EdgeAlgebra::QuadEdge* edge = it->first;
		EdgeAlgebra::Site* origin = edge->GetOrigin();
		EdgeAlgebra::Site* destination = edge->GetDestination();

		m_lines.push_back(Line(Point(origin->m_x, origin->m_y), Point(destination->m_x, destination->m_y)));
		
		++it;	
	}
}
	
void DelaunayTestLogic::CreateCircles(const std::map<EdgeAlgebra::QuadEdge*, char>& edges)
{
	std::map<EdgeAlgebra::QuadEdge*, char>::const_iterator it = edges.begin();
	std::map<EdgeAlgebra::QuadEdge*, char>::const_iterator itEnd = edges.end();
	while (it != itEnd)
	{
		EdgeAlgebra::QuadEdge* edge = it->first;		
		
		/// @todo refactor this code to a method
		EdgeAlgebra::QuadEdge* rnext = edge->GetRnext();
		if (rnext->GetRnext()->GetRnext()->GetPtr() == edge)
		{
			CreateCircle(edge, rnext);
		}

		EdgeAlgebra::QuadEdge* lnext = edge->GetLnext();
		if (lnext->GetLnext()->GetLnext()->GetPtr() == edge)
		{
			CreateCircle(edge, lnext);
		}

		++it;
	}
}

void DelaunayTestLogic::CreateCircle(EdgeAlgebra::QuadEdge* edge, EdgeAlgebra::QuadEdge* xnext)
{
	EdgeAlgebra::Site* s1 = edge->GetOrigin();
	EdgeAlgebra::Site* s2 = edge->GetDestination();
	EdgeAlgebra::Site* s3 = xnext->GetDestination();

	Point p1(s1->m_x, s1->m_y);
	Point p2(s2->m_x, s2->m_y);
	Point p3(s3->m_x, s3->m_y);

	m_circles.push_back(Circle(p1, p2, p3));
}