/// Copyright (c) 2009 Ruben Penalva. All rights reserved.
/// Redistribution and use in source and binary forms, with or without modification, 
/// are permitted provided that the following conditions are met:
///    1. Redistributions of source code must retain the above copyright notice, this 
///    list of conditions and the following disclaimer.
///    2. Redistributions in binary form must reproduce the above copyright notice,
///    this list of conditions and the following disclaimer in the documentation and/or
///    other materials provided with the distribution.
///    3. The name of the author may not be used to endorse or promote products derived
///    from this software without specific prior written permission.
///
/// THIS SOFTWARE IS PROVIDED BY THE AUTHOR ``AS IS'' AND ANY EXPRESS OR IMPLIED WARRANTIES,
/// INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR 
/// A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE AUTHOR BE LIABLE FOR ANY DIRECT, 
/// INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED 
/// TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS 
/// INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT 
/// LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF 
/// THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.

/// @file DelaunayTestLogic.h
/// This source file is part of 2dRender Delaunay Test.
/// The file holds the DelaunayTest2dRender class interface.
/// @author Ruben Penalva (ruben.penalva@rpenalva.com)
#ifndef _DELAUNAY_TEST_LOGIC_H_
#define _DELAUNAY_TEST_LOGIC_H_

#include "Point.h"
#include "Line.h"
#include "Circle.h"

#include <vector>
#include <map>

namespace EdgeAlgebra
{
class Site;
class QuadEdge;
class DelaunayTriangulation;
}

namespace DelaunayTest
{

/// @brief It handles all the logic of the test. It has the sites
/// and the triangulation. Its called by Runner, which act as a
/// controller between Logic and Render.
class DelaunayTestLogic
{
public:
	/// Constructor
	DelaunayTestLogic(void);

	/// Destructor
	~DelaunayTestLogic(void);

	/// @brief Triangulates the sites.
	void Triangulate(void);

	/// @brief Resets both sites and triangulation.
	void Reset(void);

	/// @brief Adds fixed number of sites if
	/// its not yet triangulated.
	void AddSites(void);

	/// @brief Removes fixed number of sites if
	/// its not yet triangulated
	void RemoveSites(void);

	/// @brief Enables/disables showing of circles
	/// @note Probably this needs to be on gui side
	void EnableCircles(void);

	/// @brief Gets the sites of the triangulation
	/// @return array of sites
	const std::vector<Point>& GetPoints(void) const;

	/// @brief Gets the edges of the triangulation
	/// @return array of edges
	const std::vector<Line>& GetLines(void) const;

	/// @brief Gets the circles that check correctness
	/// of delaunay triangulation
	const std::vector<Circle>& GetCircles(void) const;
	
	/// @brief Check if show circles is enabled
	/// @return true if enabled, false otherwise
	bool IsCirclesEnabled(void);

	/// @brief Check if it is triangulated
	/// @return true if its triangulated, false otherwise
	bool IsTriangulated(void);

private:
	/// Number of points
	const int m_pointsCount;

	/// Min range of position coordinate
	const float m_minRange;

	/// Max range of position coordinate
	const float m_maxRange;

	/// Fixed number of points to add
	const int m_points2AddCount;
	
	/// Maximu number of points to add
	const int m_maxPoints2Add;

	/// Points of triangulation
	std::vector<Point> m_points;

	/// Edges of triangulation
	std::vector<Line> m_lines;

	/// Circles checkin delaunay correctness
	std::vector<Circle> m_circles;

	/// Is it triangulated?
	bool m_isTriangulated;

	/// are the circles enabled?
	bool m_circlesEnabled;

	/// @note hide copy constructor from interface user
	DelaunayTestLogic(const DelaunayTestLogic&);

	/// @note hide assignment operator from interface user
	DelaunayTestLogic& operator=(const DelaunayTestLogic&);

	/// @brief Adds pointsCount random sites to the triangulation.
	/// Its range is [m_minRange, m_maxRange].
	/// @param pointsCount number of points to add
	void AddRandomPoints(int pointsCount);

	/// @brief Triangulated the points in m_points. It uses
	/// Delaunay lib (in house developed) to triangulate.
	void TriangulatePoints(void);

	/// @brief Given an array of edges it creates lines in
	/// m_lines.
	/// @param edges edges of the triangulation
	void CreateLines(const std::map<EdgeAlgebra::QuadEdge*, char>& edges);

	/// @brief Given an array of edges it creates circles in
	///	m_circles. It uses DelaunayTestLogic::CreateCircle
	/// @param edges edges of the triangulation
	/// @see DelaunayTestLogic::CreateCircle
	void CreateCircles(const std::map<EdgeAlgebra::QuadEdge*, char>& edges);

	/// @brief Given two edges it creates the circle that defines the three
	/// points in m_circles.
	/// @param edge an edge
	/// @param xnext the other edge
	void CreateCircle(EdgeAlgebra::QuadEdge* edge, EdgeAlgebra::QuadEdge* xnext);
};

}

#endif