/// Copyright (c) 2009 Ruben Penalva. All rights reserved.
/// Redistribution and use in source and binary forms, with or without modification, 
/// are permitted provided that the following conditions are met:
///    1. Redistributions of source code must retain the above copyright notice, this 
///    list of conditions and the following disclaimer.
///    2. Redistributions in binary form must reproduce the above copyright notice,
///    this list of conditions and the following disclaimer in the documentation and/or
///    other materials provided with the distribution.
///    3. The name of the author may not be used to endorse or promote products derived
///    from this software without specific prior written permission.
///
/// THIS SOFTWARE IS PROVIDED BY THE AUTHOR ``AS IS'' AND ANY EXPRESS OR IMPLIED WARRANTIES,
/// INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR 
/// A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE AUTHOR BE LIABLE FOR ANY DIRECT, 
/// INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED 
/// TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS 
/// INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT 
/// LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF 
/// THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.

/// @file DelaunayTriangulation.cpp
/// This source file is part of 2dRender Delaunay Test.
/// The file holds the DelaunayTriangulation class implementation.
/// @author Ruben Penalva (ruben.penalva@rpenalva.com)

#include <stdio.h>
#include <assert.h>
#include <vector>

#include "QuadEdge.h"
#include "Site.h"
#include "DelaunayTriangulation.h"

void OrientationTests(void)
{
	/////////////////////////////////////////
	/// QuadEdge orientation operators tests	
	/////////////////////////////////////////
	{
		EdgeAlgebra::QuadEdge edge;

		EdgeAlgebra::QuadEdge* rot = edge.GetRot();
		EdgeAlgebra::QuadEdge* sym = edge.GetSym();
		EdgeAlgebra::QuadEdge* invRot = edge.GetInvRot();

		/// Rot test
		assert(rot->GetRot() == sym);
		assert(rot->GetSym() == invRot);
		assert(rot->GetInvRot() == &edge);

		/// Sym test
		assert(sym->GetRot() == invRot);
		assert(sym->GetSym() == &edge);
		assert(sym->GetInvRot() == rot);

		/// InvRot test
		assert(invRot->GetRot() == &edge);
		assert(invRot->GetSym() == rot);
		assert(invRot->GetInvRot() == sym);
	}
}


void SiteTests(void)
{
	/////////////////////////////////////////
	/// Site tests
	/////////////////////////////////////////
	/// CounterClockWise Tests
	{
		EdgeAlgebra::Site a(0.0f, 0.0f);
		EdgeAlgebra::Site b(1.0f, 1.0f);
		EdgeAlgebra::Site c(-1.0f, 0.0f);
		EdgeAlgebra::Site d(1.0f, 0.0f);

		assert(d.CounterClockWise(&b, &a));
		assert(c.CounterClockWise(&b, &a));

		assert(c.CounterClockWise(&a, &b));
		assert(d.CounterClockWise(&a, &b));
	}

	/// InCircle Tests
	{
		EdgeAlgebra::Site a(0.0f, 1.0f);
		EdgeAlgebra::Site b(1.0f, 0.0f);
		EdgeAlgebra::Site c(0.0f, -1.0f);
		EdgeAlgebra::Site d(0.0f, 0.0f);
		EdgeAlgebra::Site e(1.0f, 1.0f);

		assert(d.InCircle(&c, &b, &a) == true);
		assert(e.InCircle(&c, &b, &a) == false);
	}
}


void DelaunayTest(void)
{
	std::vector<EdgeAlgebra::Site*> sites;
	sites.push_back(new EdgeAlgebra::Site(0.0f, 0.0f));
	sites.push_back(new EdgeAlgebra::Site(1.0f, 1.0f));
	sites.push_back(new EdgeAlgebra::Site(2.0f, 0.0f));
	sites.push_back(new EdgeAlgebra::Site(3.0f, 1.0f));

	EdgeAlgebra::DelaunayTriangulation delaunay(sites);

	std::vector<EdgeAlgebra::Site*>::iterator it = sites.begin();
	std::vector<EdgeAlgebra::Site*>::iterator itEnd = sites.end();
	while (it != itEnd)
	{
		delete *it;
		++it;
	}
}

int main(int argc, char** argv)
{
	DelaunayTest();

	return 0;
}